#!/usr/bin/tcsh -f

# Compare pm-bc with Python script.  Tested on Fedora Linux.
# Use with pm-bc compiled in NDEBUG mode; the default debug build
# yields an executable that is slow due to pma internal checks.

set niter = 1000
set pf    = persisted_data
set pmbc  = ../../LATEST_PM_BC_NDEBUG  # path of pm-bc built in NDEBUG mode

if ( ! ( -e $pmbc && -x $pmbc ) ) then
    echo "set 'pmbc' shell var to path of pm-bc executable"
    exit
endif

echo "empty loop, $niter iterations"
set begin = `date +%s.%N`
set i = 0
while ($i < $niter)
    echo "$i\n-1" > /dev/null
    @ i ++
end
echo $begin `date +%s.%N` | gawk '{print "time " $2-$1}'

echo 'pm-bc' ; set o = out.bc ; rm -f $pf $o
truncate -s 40960000 $pf
setenv BC_PM_HEAP_FILE $pf
touch $o
set begin = `date +%s.%N`
set i = 0
while ($i < $niter)
    echo "$i\n-1" | $pmbc winavg.bc >> $o
    @ i ++
end
echo $begin `date +%s.%N` | gawk '{print "time " $2-$1}'
unsetenv BC_PM_HEAP_FILE

echo 'python' ; set o = out.py ; rm -f $pf $o
setenv PYTHON_PERSIST_FILE $pf
touch $o
set begin = `date +%s.%N`
set i = 0
while ($i < $niter)
    echo "$i\n-1" | python winavg.py >> $o
    @ i ++
end
echo $begin `date +%s.%N` | gawk '{print "time " $2-$1}'
unsetenv PYTHON_PERSIST_FILE

rm -f $pf

