# PM-BC Documentation

This documentation is for the PM-BC project, which is a persistent memory version of the original BC project. For documentation of the original BC, please refer to the file `README`.

An account of PM-BC will be published in the Nov/Dec 2024 issue of ACM _Queue_ magazine.

Updates to PM-BC may be published at the following URL:

https://web.eecs.umich.edu/~tpkelly/pm-bc/

## Building PM-BC

The building process of PM-BC is the same as that of the original BC:
```bash
autoreconf --install
automake --add-missing
./configure
make
```

Note that the commands above compile the persistent memory allocator (`pma`) in debug mode (the default), which performs extensive internal sanity checks and is therefore slow.  For a faster `pm-bc` executable, set the environment variable `CFLAGS` to the value `-DNDEBUG` before running the commands above.

For cleaning up the build files, you can either run `make clean` or `make distclean`. If former, you can rebuild the project with `make` without running the previous commands; if latter, you need to run the previous commands again.

## Running PM-BC

PM-BC adds a persistent memory mode to the original BC where other functionalities are kept the same. To run PM-BC in persistent memory mode, you need to specify the path to the persistent memory file as the `BC_PM_HEAP_FILE` environment variable; otherwise, it will run in the original mode.

The following is an example of running PM-BC in persistent memory mode:
(Assume the path to the `bc` executable is `./bc/bc`, and that to the persistent heap file is `heap.pma`)
```bash
$ truncate -s 4096000 heap.pma    # Create a 4MB persistent memory file
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -q <<< "a=2;a"
2
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -q <<< "a=a+1;a"
3
```
and the following is an example of running PM-BC in the original mode:
```bash
$ ./bc/bc -q <<< "a=2;a"
pma.c:405: Warning: no backing file provided; falling back on standard malloc
2
$ ./bc/bc -q <<< "a=a+1;a"
pma.c:405: Warning: no backing file provided; falling back on standard malloc
1
```

## Notes on the "-l" Option

The `-l` option is already present in the original BC, its functionality is to enable users to use predefined math routines. If the user wants to use the `-l` option in non-persistent memory mode in PM-BC, they may ignore this message.

However, if the user wants to use the `-l` option in persistent memory mode in PM-BC, they need to ensure that `-l` is invoked only once per session. If the user invokes `-l` second or more times in the same session, the program will terminate with an error message.

The enabling of BC's predefined math routines is implemented by loading a math library (`bc/libmath.b`). When the user invokes `-l` for the second or more times, the program will try to load the math library again. This creates risks of the math library functions' definitions overriding the user-defined functions' definitions in the persistent memory file, if they have the same names. To avoid this, the program terminates when the user tries to load the math library again.

To summarize, if the user wants to use the `-l` option in persistent memory mode in PM-BC, they should invoke it in the first PM-BC invocation only.

The following is an example of running PM-BC in non-persistent memory mode with the `-l` option:
```bash
$ ./bc/bc -q <<< "1+1"
pma.c:405: Warning: no backing file provided; falling back on standard malloc
2
$ ./bc/bc -ql <<< "1+1"
pma.c:405: Warning: no backing file provided; falling back on standard malloc
2
$ ./bc/bc -ql <<< "1+1"
pma.c:405: Warning: no backing file provided; falling back on standard malloc
2
```
and the following is an example of running PM-BC in persistent memory mode with the `-l` option:
```bash
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -ql <<< "1+1"
2
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -q <<< "1+1"
2
```
The follow are incorrect examples of running PM-BC in persistent memory mode with the `-l` option:
```bash
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -q <<< "1+1"
2
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -ql <<< "1+1"
bc: -l is not allowed starting from second invocation in one session in persistent mode.
```
```bash
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -ql <<< "1+1"
2
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -q <<< "1+1"
2
$ BC_PM_HEAP_FILE=heap.pma ./bc/bc -ql <<< "1+1"
bc: -l is not allowed starting from second invocation in one session in persistent mode.
```

## Notes on the "strdup()" Macro

In `lib/pmactrl.h`, a macro named `strdup()` is defined to replace the C standard library's `strdup()` function. The purpose is to make sure all `strdup()` calls copies strings persistently. A side effect of this is that ALL `strdup()` calls in source files that directly or indirectly include `lib/pmactrl.h` will be replaced by this macro. Developers or maintainers should consider workarounds to avoid this macro definition if they want to use the original `strdup()` function.
