/* pmactrl.h: Controls how bc uses the persistent memory allocator in pma.c and pma.h. */
/*
    Copyright (C) 2023, 2024 Ziheng (Aaron) Su

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License , or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to:

      The Free Software Foundation, Inc.
      51 Franklin Street, Fifth Floor
      Boston, MA 02110-1301  USA


    You may contact the author of this file, Ziheng (Aaron) Su, by:
       e-mail:  aaronsu2003@gmail.com

*************************************************************************/

#ifndef PMACTRL_H
#define PMACTRL_H

#include "pma.h"

#define malloc pma_malloc
#define calloc pma_calloc
#define realloc pma_realloc

#ifndef USE_FREE
#define free pma_free
#endif // USE_FREE

#define strdup(src) ({ \
    char *s   = (src), \
         *dst = s ? pma_malloc(strlen(s) + (size_t)1) : NULL; \
    (s && dst) ? strcpy(dst, s) : NULL; \
})

extern struct root_pointer *root_pointer;  /* single entry point to the persistent memory */

#ifndef NOT_USE_ROOT_POINTER  /* When not defined, replace global variables with their corresponding root pointer versions. */

#define break_label    root_pointer->break_label
#define if_label       root_pointer->if_label
#define continue_label root_pointer->continue_label
#define next_label     root_pointer->next_label
#define functions      root_pointer->functions
#define f_names        root_pointer->f_names
#define f_count        root_pointer->f_count
#define variables      root_pointer->variables
#define v_names        root_pointer->v_names
#define v_count        root_pointer->v_count
#define arrays         root_pointer->arrays
#define a_names        root_pointer->a_names
#define a_count        root_pointer->a_count
#define ex_stack       root_pointer->ex_stack
#define fn_stack       root_pointer->fn_stack
#define i_base         root_pointer->i_base
#define o_base         root_pointer->o_base
#define scale          root_pointer->scale
#define next_array     root_pointer->next_array
#define next_func      root_pointer->next_func
#define next_var       root_pointer->next_var
#define name_tree      root_pointer->name_tree

#endif // NOT_USE_ROOT_POINTER

#endif // PMACTRL_H
